import clingo

def solve(AF, n, semantics):
    """ Given an AF, returns n extensions according to the semantics
    passed as input. If n=0, then returns all the extensions."""

    asp = AF + semantics + "#show in/1."

    control = clingo.Control()
    control.add("base", [], asp)
    control.ground([("base", [])])

    control.configuration.solve.models = n

    models = []

    def on_model(model):
        nonlocal models
        m = []
        for atom in model.symbols(shown=True):
            # in(a3) -> 3
            m.append(int(str(atom.arguments[0])[1:]))

        models.append(m)

    answer = control.solve(on_model=on_model)
    return models

def solve_adm(AF, n):
    """ Given an AF, returns n admissible extensions (if n=0, returns all) """
    semantics = """
    in(X) :- not out(X), arg(X).
    out(X) :- not in(X), arg(X).

    :- in(X), in(Y), att(X,Y).

    defeated(X) :- in(Y), att(Y,X).

    not_defended(X) :- att(Y,X), not defeated(Y).

    :- in(X), not_defended(X)."""

    return solve(AF, n, semantics)

def solve_pref(AF, n):
    """ Given an AF, returns n preferred extensions (if n=0, returns all) """
    semantics = """
    in(X) :- not out(X), arg(X).
    out(X) :- not in(X), arg(X).

    :- in(X), in(Y), att(X,Y).

    defeated(X) :- in(Y), att(Y,X).
    not_defended(X) :- att(Y,X), not defeated(Y).

    :- in(X), not_defended(X).

    not_trivial :- out(X).
    ecl(X) : out(X) :- not_trivial.
    spoil | ecl(Z) : att(Z,Y) :- ecl(X), att(Y,X).
    spoil :- ecl(X), ecl(Y), att(X,Y).
    spoil :- in(X), ecl(Y), att(X,Y).
    ecl(X) :- spoil, arg(X).
    :- not spoil, not_trivial.
    """

    return solve(AF, n, semantics)

def solve_comp(AF, n):
    """ Given an AF, returns n complete extensions (if n=0, returns all) """
    semantics = """
    in(X) :- not out(X), arg(X).
    out(X) :- not in(X), arg(X).

    :- in(X), in(Y), att(X,Y).

    defeated(X) :- in(Y), att(Y,X).

    not_defended(X) :- att(Y,X), not defeated(Y).

    :- in(X), not_defended(X).

    :- out(X), not not_defended(X)."""

    return solve(AF, n, semantics)


def solve_cf(AF, n):
    """ Given an AF, returns n admissible extensions (if n=0, returns all) """
    semantics = """
    in(X) :- not out(X), arg(X).
    out(X) :- not in(X), arg(X).

    :- in(X), in(Y), att(X,Y)."""

    return solve(AF, n, semantics)