import pickle
import numpy as np
import matplotlib.pyplot as plt
import sys

# If the command line argument is unsupplied, by default we plot the results
# in the experiments.pkl file.

file = sys.argv[1]

with open(file, "rb") as f:
    results = pickle.load(f)

# First, we get the dispersions and functions and ks considered
dispersions = results["dispersions"]
functions = results["functions"]
representationSizes = results["representationSizes"]

# This is for plotting reasons,
# it is the x-axis (corresponds to the number of dispersions used)
ticks = range(len(dispersions))

# to pretty-print function names

name = {
    "util": "Utilitarian",
    "greedy_util": "Utilitarian (greedy)",
    "harmonic": "Harmonic",
    "greedy_harmonic": "Harmonic (greedy)",
    "egal": "Egalitarian",
    "greedy_egal": "Egalitarian (greedy)",
}

# Generate the plots. 
# For representation type and metric,

for function in functions:
    score = results[function]

    for representationType in ["all", "defCore"]:

        # we make a plot. We plot all functions:

        for representationSize in representationSizes:

            # for each value of dispersion, we compute
            # the average outcome-score obtained by this value of k, with this type of representation 

            res = []
            for dispersion in dispersions:
                res.append(np.mean(score[representationSize][representationType][dispersion]))

            # plot: dispersion on x-axis, and average outcome-score on y-axis

            plt.plot(ticks, res, label=f"k={representationSize}")

        # ok, plotted al functions: Now we show the plot

        # Adding labels and title
        plt.xlabel('dispersion value')
        plt.ylabel(name[function] + " score")
        plt.title("representation: " + ("Regular" if representationType == "all" else "Defendable Core"))

        # Adding a legend to identify each curve
        plt.legend()

        # axis: add the dispersions value
        plt.xticks(ticks, (f"{dispersion:.1f}" for dispersion in dispersions))

        # Display the plot
        plt.show()